/**
 * Copyright 2008, Daniel Molina Cabrera <danimolina@gmail.com>
 * 
 * This file is part of software Realea
 * 
 * Realea is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Realea is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _MA2LSIA_H

#define _MA2LSIA_H 1

#include "hybrid.h"
#include "restart.h"
#include "selectls.h"
#include "lsparammem.h"

namespace realea {

/**
 * @class MA2LSI
 *
 * @brief memetic algorithm following the model proposed in the Daniel Molina's thesis
 */
class MA2LSIA : public Hybrid {
public:
	/**
	 * Constructor
	 *
	 * 
	 */
	MA2LSIA(IEAlgorithm *alg, ILocalSearch *ls);

	~MA2LSIA(void);
	/**
	 * @param ratio. Set the global ratio invested into the Local Search
	 *
	 * @param ratio global ls/total ratio
	 * (change the intensity, because only one LS improvement is made)
	 */
	void setEffortRatio(double ratio);
	void setDebug(void);

	void recoverIndividual(unsigned oldind, tGen *aind, unsigned size, tGen *aoptional, unsigned size_optional);
	void storeIndividual(tIndividualRealPtr ind, tGen **paind, unsigned *pmax, tGen **padditional, unsigned *pmaxad); 

	void setMaxEval(unsigned int maxeval);

	void setSelectImprovementStrategy(SelectImprovementLS *select_improvement) {
	    m_select_improvement = select_improvement;
	}

	void setRunning(Running *running); 

	/**
	 * Set the restart strategy. If it is not called never it will make a restart.
	 */
	void setRestart(RestartStrategy *restart, int maxfailed=-1);

	/**
	 * Alter the EA and after the LS, then the EA again, the LS, ...
	 */
	unsigned realApply(tChromosomeReal &sol, tFitness &fitness);
	unsigned init(void);

	void setInitEval(IEval* eval); 

private:
/**
 * Calculate the new frec to apply to obtain the indicated ratio.
 *
 * It is required because there are AEs that could not follow the frec indicated in the previous step
 *
 * @param nevalalg total number evaluation during the EA
 * @param nevalls total number evaluation during the LS
 * @param intensity LS intensity
 *
 * @return frec to use the EA to maintain the ratio
 */
    unsigned calculateFrec(double nevalalg, double nevalls, unsigned intensity, unsigned neval, double ratio);
    bool hasDiversity(PopulationReal *pop); 
    void setDif(bool debug, string ident, unsigned id, tFitness oldfit, tFitness newfit);
    void updateFrec(double effort, double effort_obtained); 

private:
	internal::LSParametersMemory *m_memory;
	unsigned m_frec_update;
	unsigned m_nevalalg;
	unsigned m_nevalls;
	unsigned m_maxfailed;
	IEvalInd *m_initeval;
	unsigned m_initMaxEval;
	double m_effort;
	RestartStrategy *m_restart;
	SelectImprovementLS *m_select_improvement; 
	bool m_debug;
};


};

#endif
