/**
 * Copyright 2008, Daniel Molina Cabrera <danimolina@gmail.com>
 * 
 * This file is part of software Realea
 * 
 * Realea is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Realea is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "problemtablefactory.h"
#include <cassert>

using namespace realea;

ProblemTableFactory::ProblemTableFactory(unsigned maxeval, FUNCTION *fun, unsigned max) {
    m_fun = fun;
    m_max = max;
    m_maxeval = maxeval;
}

ProblemTableFactory::~ProblemTableFactory(void) {
    m_fun = NULL;
};

ProblemPtr ProblemTableFactory::get(unsigned int ident) {
    ProblemPtr prob (new Problem());

    assert(ident >= 1 && ident <= m_max);

    // Obtengo la función
    FUNCTION *fun = &(m_fun[ident-1]);
    // Inicio el problema
    unsigned ndim = fun->genes;
    prob->setDimension(ndim);
    double min = fun->range[0];
    double max = fun->range[1];

    for (unsigned i = 0; i < ndim; ++i) {
	prob->setDomainValues(i, min, max, true);
    }

    prob->setOptimize(fun->optime, 0);
    prob->setMaxEval(m_maxeval);
    prob->setMinimize();

    prob->setEval(fun->eval);

    return prob;
}
