/**
 * Copyright 2008, Daniel Molina Cabrera <danimolina@gmail.com>
 * 
 * This file is part of software Realea
 * 
 * Realea is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Realea is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _PROBLEM_TABLE_FACTORY_H

#define _PROBLEM_TABLE_FACTORY_H 1

#include "problemfactory.h"

namespace realea {

/**
 * @file La idea de la clase ProblemTableFactory es facilitar la creación de 
 * benchmark
 * 
 * La clase FUNCTION permite especificar los parámetros necesarios para
 * una función de evaluación.
 */
struct FUNCTION {
   int ident; 		/**< Parámetro identificador de la función */
   int genes; 		/**< Dimensión de las soluciones */
   string name; 	/**< Nombre de la función (para poder mostrarla en pantalla */
   tEval eval;  	/**< Puntero a la función de evaluación */
   tGen range[2];	/**< Guarda los valores mínimos y máximos para dicha función */
   tGen initrange[2];	/**< Guarda los valores mínimos y máximos de la inicialización dicha función */
   double optime;    	/**< Valor óptimo */
};

/**
 * @class Permite crear fácilmente clases de tipo ProblemFactory para un conjunto
 * de funciones. 
 */
class ProblemTableFactory : public ProblemFactory {
   public:
       /**
	* Asigna la tabla de funciones a la clase. Nunca se borrará, para poder
	* usar una tabla estática
	*
	* @param maxeval maximum eval
	* @param fun tabla de funciones
	* @param max vector de elementos
	*/
       ProblemTableFactory(unsigned maxeval, FUNCTION *fun, unsigned max);
       ~ProblemTableFactory(void);
       ProblemPtr get(unsigned int ident);
    private:
       FUNCTION *m_fun;
       unsigned m_max;
       unsigned m_maxeval;
};

}
#endif
