#include <cmath>
#include "F7-F11.h"
#include "f7f11data.h"

tFitness f_Schwefel2_22(int dim, double *s) 
{
    tFitness sum, currentGen, prod;

    sum = 0.0;
    prod = 1.0;

    for (int i = 0; i < dim; i++) 
    {
        currentGen = fabs(s[i]-f7[i]);
        sum += currentGen;
        prod *= currentGen;
    }

    return sum + prod;
}

tFitness f_Schwefel1_2(int dim, double *s)
{
   	tFitness Sum=0.0, Val=0.0;

	for (int i = 0; i < dim; i++)
    {  
        Val += s[i]-f8[i];
	    Sum += Val * Val;
    }

	return Sum;
}

tFitness f_10(double x, double y)
{
     double p, z, t;

     p=(x*x+y*y);
     
     z=pow(p, 0.25);
     t=sin(50.0*pow(p, 0.1));
     t=t*t+1.0;
     
     return z*t;
}

tFitness Extended_f_10(int dim, double *x)
{
     double suma=0.0;

     for(int i=0; i<dim-1; i++)
           suma+=f_10(x[i]-f9[i], x[i+1]-f9[i+1]);

     suma+=f_10(x[dim-1]-f9[dim-1], x[0]-f9[0]);

     return suma;
}

tFitness f_Bohachevsky(int dim, double *s) 
{   
    const double PI = 3.141592653589793;
    tFitness sum = 0.0;
    int i;
    double currentGen;
    double nextGen;

    currentGen = s[0]-f10[0];

    for (i = 1; i < dim; i++) 
    {
        nextGen = s[i]-f10[i];
        sum += currentGen * currentGen + 2.0 * nextGen * nextGen;
        sum += -0.3 * cos(3.0 * PI * currentGen) -0.4 * cos(4.0 * PI * nextGen) + 0.7;
        currentGen = nextGen;
    }

    return sum;
}

tFitness f_Schaffer(int dim, double *s) 
{
    int i;
    tFitness sum;
    double aux, aux2;
    double currentGen, nextGen;

    sum = 0.0;
    currentGen = s[0]-f11[i];
    currentGen = currentGen * currentGen;

    for (i = 1; i < dim; i++) 
    {
        nextGen = s[i]-f11[i];
        nextGen = nextGen * nextGen;
        aux = currentGen + nextGen;
        currentGen = nextGen;
        aux2 = sin(50. * pow(aux, 0.1));
        sum += pow(aux, 0.25) * (aux2 * aux2 + 1.0);
    }

    return sum;
}
