#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <cmath>

#include "funcec08.h"
#include "cec08data.h"
#include <cassert>

using namespace std;

#define abss(a)     (a<0 ? (-a) : a)
#define max(a , b)  (a>b ?   a  : b)
#define min(a , b)  (a>b ?   b  : a)


static double pi;
static double e;
int gfunc_number;
static double m_optime;

void init_cec2008(int func_number) {
	pi = acos(-1.0);
	e = exp(1.0);
	gfunc_number = func_number;
}

void finish_cec2008(void) {

}


double Shifted_Sphere( int dim , const double* x ){
    int i;
    double z;
    double F = 0;
    for(i=0;i<dim;i++){
        z = x[i] - sphere[i];
        F += z*z;
    }
    return F + f_bias[0];
}

double Schwefel_Problem( int dim , const double* x ){
    int i;
    double z;
    double F = abss(x[0]);
    for(i=1;i<dim;i++){
    	  z = x[i] - schwefel[i];
        F = max(F , abss(z));
    }
    return F + f_bias[1]; 
}

double Shifted_Rosenbrock( int dim , const double* x ){
    int i;
    double z[dim];
    double F = 0;

    for(i=0;i<dim;i++) z[i] = x[i] - rosenbrock[i] + 1;   

    for(i=0;i<dim-1;i++){    
        F = F + 100*( pow((pow(z[i],2)-z[i+1]) , 2) ) + pow((z[i]-1) , 2);
    }
    return F + f_bias[2]; 
}


double Shifted_Rastrigin( int dim , const double* x )
{
    int i;
    double z;
    double F = 0;
    for(i=0;i<dim;i++){  
        z = x[i] - rastrigin[i];
        F = F + ( pow(z,2) - 10*cos(2*pi*z) + 10);
    }
    return F + f_bias[3]; 
}

double Shifted_Griewank( int dim , const double* x ){
    int i;
    double z;
    double F1 = 0;
    double F2 = 1;
    for(i=0;i<dim;i++){       
        z = x[i] - griewank[i];
        F1 = F1 + ( pow(z,2) / 4000 );
        F2 = F2 * ( cos(z/sqrt(i+1)));

    }
    return (F1 - F2 + 1 + f_bias[4]); 
}

double Shifted_Ackley( int dim , const double* x ){
    int i;
    double z;
    double Sum1 = 0;
    double Sum2 = 0;
    double F = 0;
    for(i=0;i<dim;i++){   
        z = x[i] - ackley[i];
        Sum1 = Sum1 + pow(z , 2 );
        Sum2 = Sum2 + cos(2*pi*z);
    }
    F = -20*exp(-0.2*sqrt(Sum1/dim)) -exp(Sum2/dim) + 20 + e + f_bias[5];

    return F; 
}
void getInfo_cec2008(unsigned fun, string &name, double &min, double &max, double &optime) {
    if (fun == 1) {
	name = "Shifted_Sphere";
	min = -100;
	max = 100;
	optime = -450.0;
    }
    else if (fun == 2) {
	name = "Schwefel_Problem";
	min = -100;
	max = 100;
	optime = -450.0;
    }
    else if (fun == 3) {
	name = "Shifted_Rosenbrock";
	min = -100;
	max = 100;
	optime = 390.0;
    }
    else if (fun == 4) {
	name = "Shifted_Rastrigin";
	min = -5;
	max = 5;
	optime = -330.0;
    }
    else if (fun == 5) {
	name = "Shifted_Griewank";
	min = -600;
	max = 600;
	optime = -180.0;
    }
    else if (fun == 6) {
	name = "Shifted_Griewank";
	min = -32;
	max = 32;
	optime = -140.0;
    }
    else if (fun == 7) {
       name = "Java FastFractal";
       min = -1;
       max = 1;
       optime = -15000;
    }


    m_optime = optime;
}

double eval_cec2008(const tGen *x, int dim) {
   assert(gfunc_number >= 0 && gfunc_number <= 6);

    switch(gfunc_number){
        case 1 : return Shifted_Sphere(dim , x ) - m_optime;
            break;
        case 2 : return Schwefel_Problem(dim , x )- m_optime;
            break;
        case 3 : return Shifted_Rosenbrock(dim , x )- m_optime;
            break;
        case 4 : return Shifted_Rastrigin(dim , x )- m_optime;
            break;
        case 5 : return Shifted_Griewank(dim , x )- m_optime;
            break;
        case 6 : return Shifted_Ackley(dim , x ) - m_optime;
            break;
	
	default: return -1;
    }

}
