#include "problembbob2009.h"
#include <cassert>
#include <cstdio>
#include <cstring>
//#include "bbob2009/bbobStructures.h" /* Include all declarations for BBOB calls */

using namespace realea;

ProblemBBOB2009::ProblemBBOB2009(string algname, string resultpath, unsigned int dim, int instance) {
    assert(dim == 2 || dim == 3 || dim == 5 
		|| dim == 10 || dim == 20 || dim == 40);
    assert(instance >= 1 && instance <= 5);

    m_ndim = dim;
    m_instance = instance;
    m_init = false;
    m_algname = algname;
    m_resultpath = resultpath;
}

ProblemBBOB2009::~ProblemBBOB2009(void) {
    if (m_init) {
       /* call the BBOB closing function to wrap things up neatly */
       fgeneric_finalize();
    }
}

void ProblemBBOB2009::init(unsigned int fun) {
    if (m_init) {
	return;
    }

    assert((fun >= 1 && fun <= 24) // (noiseless)
		|| (fun >= 101 && fun <= 130) //(noisy)
	    );

    /**************************************************
     *          BBOB Mandatory initialization         *
     *************************************************/
    /* To make the noise deterministic. */
    fgeneric_noiseseed(30);
    /* retrieve all default parameters of BBOB calls  */
    ParamStruct params = fgeneric_getDefaultPARAMS();

    /* modify the following parameters, choosing a different setting
     * for each new experiment */
    strcpy(params.dataPath, m_resultpath.c_str());  /* different folder for each experiment! */
    /* please beforehand run from the command-line 'python createfolders.py PUT_MY_BBOB_DATA_PATH'
     * to create the necessary folder structure to run an experiment. */
    strcpy(params.algName, m_algname.c_str());
    strcpy(params.comments, "RatioBL=0.5,Intensity=500");

    /* set DIM, funcId, instanceId to initialize BBOB fgeneric */
    params.DIM = m_ndim;
    params.funcId = fun;
    params.instanceId = m_instance;
    /* call the BBOB initialization */
    fgeneric_initialize(params);
    m_init = true;
}

tFitness eval_bbob2009(const tGen *x, int ndim) {
    double sol[ndim];

    for (int i = 0; i < ndim; i++) {
	sol[i] = x[i];
    }
    return fgeneric_evaluate(sol);
}

ProblemPtr ProblemBBOB2009::get(unsigned int fun) {
    // Limpio si se pidió alguna otra función anteriormente
    init(fun);
    ProblemPtr prob (new Problem());
    prob->setDimension(m_ndim);
    
    // Set the initial range [-4,4] but it does not check the bound
    for (unsigned i = 0; i < m_ndim; ++i) {
	prob->setDomainValues(i, -5, 5, true);
    }

    // Defino el criterio de óptimo
    prob->setOptimize(fgeneric_ftarget(), 0);
    prob->setMinimize();

    // Defino la función de fitness
    prob->setEval(eval_bbob2009);

    // By default is 1000*D*D
    prob->setMaxEval(5000*m_ndim*m_ndim);
    prob->setMinimize();

    return prob;
}
