/**
 * Copyright 2008, Daniel Molina Cabrera <danimolina@gmail.com>
 * 
 * This file is part of software Realea
 * 
 * Realea is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Realea is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "funcec08.h"
#include "problemcec2008f6.h"

using namespace realea;

double eval_cec2008(const tGen *x, int dim); 


ProblemCEC2008F6::ProblemCEC2008F6(unsigned int dim) {
    if (dim > 1000) {
	dim = 1000;
    }

    m_ndim = dim;
    m_init = false;
}

ProblemCEC2008F6::~ProblemCEC2008F6(void) {
    if (m_init) {
	finish_cec2008();
    }
}


void ProblemCEC2008F6::init(unsigned int fun) {
     if (m_init) {
	finish_cec2008();
	m_init = false;
    }

    init_cec2008(fun);
    m_init = true;
}

tFitness eval_pobcec2008(const tGen *x, int dim) {
   double result = eval_cec2008(x, dim);
   return result;
}


ProblemPtr ProblemCEC2008F6::get(unsigned int fun) {
    string name;
    double min, max, optime;

    // Limpio si se pidió alguna otra función anteriormente
    init(fun);
    getInfo_cec2008(fun, name, min, max, optime);

    ProblemPtr prob (new Problem());
    prob->setDimension(m_ndim);
    
    for (unsigned i = 0; i < m_ndim; ++i) {
	prob->setDomainValues(i, min, max, true);
    }

    // Defino el criterio de óptimo
    prob->setOptimize(0, 0);
    prob->setMaxEval(5000*m_ndim);
    prob->setMinimize();


    // Defino la función de fitness
    prob->setEval(eval_pobcec2008);
    return prob;
}
