/**
 * Copyright 2008, Daniel Molina Cabrera <danimolina@gmail.com>
 * 
 * This file is part of software Realea
 * 
 * Realea is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Realea is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "problemisda.h"
#include "benchmark.h"
#include "F7-F11.h"
#include <cstdlib>
#include <cstdio>

using namespace realea;

tFitness eval_cec2008(const tGen *x, int dim); 

static unsigned m_fun;
static tFitness m_optime;

ProblemISDA::ProblemISDA(unsigned int dim) {
    if (dim > 1000) {
	dim = 1000;
    }

    m_ndim = dim;
}

ProblemISDA::~ProblemISDA(void) {
}


tFitness eval_isda(const tGen *s, int dim) {
    double x[dim];

    for (int i = 0; i < dim; i++) {
	x[i] = s[i];
    }


    if (m_fun >= 1 && m_fun <= 6) {
	return eval_func(m_fun, dim, x)-m_optime;
    }
    else if (m_fun == 7) {
	return f_Schwefel2_22(dim, x);
    }
    else if (m_fun == 8) {
	return f_Schwefel1_2(dim, x);
    }
    else if (m_fun == 9) {
	return Extended_f_10(dim, x);
    }
    else if (m_fun == 10) {
	return f_Bohachevsky(dim, x);
    }
    else if (m_fun == 11) {
	return f_Schaffer(dim, x);
    }
    else {
	fprintf(stderr, "Error: fun no valido\n");
	exit(1);
    }
}

ProblemPtr ProblemISDA::get(unsigned int fun) {
    string name;
    double min, max, optime;

    m_fun = fun;

    if (fun == 1) {
	min = -100; max = 100;
	optime = -450;
    }
    else if (fun == 2) {
       min = -100; max = 100;
       optime = -450;
    }
    else if (fun == 3) {
       min = -100; max = 100;
       optime = 390;
    }
    else if (fun == 4) {
       min = -5; max = 5;
       optime = -330;
    }
    else if (fun == 5) {
       min = -600; max = 600;
       optime = -180;
    }
    else if (fun == 6) {
       min = -32; max = 32;
       optime = -140;
    }
    else if (fun == 7) {
       min = -10; max = 10;
       optime = 0;
    }
    else if (fun == 8) {
       min = -65.536; max = 65.536;
       optime = 0;
    }
    else if (fun == 9) {
       min = -10; max = 10;
       optime = 0;
    }
    else if (fun == 10) {
       min = -15; max = 15;
       optime = 0;
    }
    else if (fun == 11) {
       min = -100; max = 100;
       optime = 0;
    }

    ProblemPtr prob (new Problem());
    prob->setDimension(m_ndim);
    
    for (unsigned i = 0; i < m_ndim; ++i) {
	prob->setDomainValues(i, min, max, true);
    }
 
    m_optime = optime;
    // Defino el criterio de óptimo
    prob->setOptimize(0, 0);
    prob->setMaxEval(5000*m_ndim);
    prob->setMinimize();

    // Defino la función de fitness
    prob->setEval(eval_isda);
    return prob;
}
