/**
 * Copyright 2008, Daniel Molina Cabrera <danimolina@gmail.com>
 * 
 * This file is part of software Realea
 * 
 * Realea is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Realea is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "problemsoft.h"
#include <realea/common/real.h>
#include <limits>
#include "benchmark.h"
#include <cstdlib>
#include <cstdio>
#include "funsoft.h"

using namespace realea;

tFitness eval_softcomputing(const tGen *x, int dim); 

static unsigned m_fun;
static tFitness m_optime;

ProblemSoftComputing::ProblemSoftComputing(unsigned int dim) {
    if (dim > 1000) {
	dim = 1000;
    }

    m_ndim = dim;
}

ProblemSoftComputing::~ProblemSoftComputing(void) {
}


tFitness eval_softcomputing(const tGen *s, int dim) {
    double x[dim];
    tFitness result;

    for (int i = 0; i < dim; i++) {
	x[i] = s[i];
    }

    if (m_fun == 1) {
	result=Shifted_Sphere(dim, (const double *) x)-m_optime;
    }
    else if (m_fun == 2) {
       result=Schwefel_Problem(dim, (const double *)x)-m_optime;
    }
    else if (m_fun == 3) {
	result=Shifted_Rosenbrock(dim, (const double *) x)-m_optime;
    }
    else if (m_fun == 4) {
	result=Shifted_Rastrigin(dim, (const double *) x)-m_optime;
    }
    else if (m_fun == 5) {
	result=Shifted_Griewank(dim, (const double *) x)-m_optime;
    }
    else if (m_fun == 6) {
	result=Shifted_Ackley(dim, (const double *) x)-m_optime;
    }
    else if (m_fun == 7) {
//	result=f_Schwefel2_22(dim, x)/1e300;
	result=f_Schwefel2_22(dim, x);
    }
    else if (m_fun == 8) {
	result=f_Schwefel1_2(dim, x);
    }
    else if (m_fun == 9) {
	result=Extended_f_10(dim, x);
    }
    else if (m_fun == 10) {
	result=f_Bohachevsky(dim, x);
    }
    else if (m_fun == 11) {
	result=f_Schaffer(dim, x);
    }
    else if (m_fun == 12) {
	result=f_Hybrid_12(dim, x);
    }
     else if (m_fun == 13) {
	result=f_Hybrid_13(dim, x);
    }
      else if (m_fun == 14) {
	result=f_Hybrid_14(dim, x);
    }
    else if (m_fun == 15) {
	result=f_Hybrid_15(dim, x);
    }
    else if (m_fun == 16) {
	result=f_Hybrid_16new(dim, x);
    }
    else if (m_fun == 17) {
	result=f_Hybrid_17new(dim, x);
    }
    else if (m_fun == 18) {
	result=f_Hybrid_18new(dim, x);
    }
    else if (m_fun == 19) {
	result=f_Hybrid_19new(dim, x);
    }
    else {
	fprintf(stderr, "Error: fun no valido\n");
	exit(1);
    }

    /*
    if (result > numeric_limits<double>::max() ) {
	result = numeric_limits<double>::max();
    }
    */

    return result;
}

ProblemPtr ProblemSoftComputing::get(unsigned int fun) {
    string name;
    double min, max, optime;

    m_fun = fun;

    optime = 0;

    if (fun == 1) {
	min = -100; max = 100;
	optime = -450;
    }
    else if (fun == 2) {
       min = -100; max = 100;
       optime = -450;
    }
    else if (fun == 3) {
       min = -100; max = 100;
       optime = 390;
    }
    else if (fun == 4) {
       min = -5; max = 5;
       optime = -330;
    }
    else if (fun == 5) {
       min = -600; max = 600;
       optime = -180;
    }
    else if (fun == 6) {
       min = -32; max = 32;
       optime = -140;
    }
    else if (fun == 7) {
       min = -10; max = 10;
    }
    else if (fun == 8) {
       min = -65.536; max = 65.536;
    }
    else if (fun == 9) {
       min = -10; max = 10;
    }
    else if (fun == 10) {
       min = -15; max = 15;
    }
    else if (fun == 11) {
       min = -100; max = 100;
    }
    else if (fun == 10) {
       min = -15; max = 15;
    }
    else if (fun == 11) {
       min = -100; max = 100;
    }
    else if (fun == 12 || fun == 13 || fun == 16 || fun == 18 || fun == 19) {
       min = -100; max = 100;
    }
    else if (fun == 14 || fun == 20) {
	min = -5; max = 5;
    }
    else if (fun == 15 || fun == 21) {
	min = -10; max = 10;
    }
    else if (fun == 17) {
	min = -100; max = 100;
    }
    else {
	fprintf(stderr, "Error[domain]: fun no valido\n");
	exit(1);
    }


    ProblemPtr prob (new Problem());
    prob->setDimension(m_ndim);
    
    for (unsigned i = 0; i < m_ndim; ++i) {
	prob->setDomainValues(i, min, max, true);
    }
 
    m_optime = optime;
    // Defino el criterio de óptimo
    prob->setOptimize(0, 0);
    prob->setMaxEval(5000*m_ndim);
    prob->setMinimize();

    // Defino la función de fitness
    prob->setEval(eval_softcomputing);
    return prob;
}
